function logimap1_evol
% ============================================================= 65
% function logimap1_evol                             2021/11/29
% Evolution of the logistic map y = r*x*(1-x) as r grows, with
% cobwebs to illustrate the bifurcations up to the period two
% solution loosing stability. Recall:
% r = r0 = 1.           Transcritical bifurcation, with new
%                       fixed point at x = 1-1/r occurs.
% r = r1 = 3.           Period 2 solution appears;
%                       x = 1-1/r loses stability.
% r = r2 ~ 3.44948...   Period doubling;
%   = 1+sqrt(6)         period 4 solution appears;
% r = r3 ~ 3.54409...   Period doubling;
%                       period 8 solution appears;
% r_inf ~ 3.569945...
%
% Three cycle starts at r = 1 + 2*sqrt(2) ~ 3.828427...
% ============================================================= 65
LiW = 'LineWidth';   LW = 2;   FtS = 'FontSize';
x = (0:100)/100;
r0 = 1;   r1 = 3;   r2 = 1+sqrt(6);   r3 = 3.54409;
r4 = 3.56; % FAKE r4
%
rinf = 3.569945;    rper3 = 1 + 2*sqrt(2);
%
r = [0.75, (r0+r1)/2, (r1+r2)/2, (r2+r3)/2, r3+0.007];
r = [r, rinf+0.00001, rper3];
% -------------------------------- Start.
N   = length(r);   n=1;   GO = 1;
%
while GO > 0;
  y = logimap(x, r(n));
  plot(x, y, '-b', LiW, LW); hold on; grid on; zoom on;
  plot([0, 1], [0, 1], '-r', LiW, LW);
  TxT = 'Map and cobweb for r = ';
  title(['{\bf ', TxT, num2str(r(n)), '}'], FtS, 18)
  xlabel('{\bf x}', FtS, 18)
  ylabel('{\bf y}', FtS, 18, 'Rotation', 0)
  xa = logimap(0.5, r(n), 100); % Point in attractor.
%
% Here goes the various alternatives!
%
if r(n) < r0;
  LMcobweb(0.5, 4, r(n));
  TxT = ['   For r < r0 = ', num2str(r0)];
  fprintf([TxT, ', x = 0 is the attractor. \n'])
end
if (r0 < r(n))*(r(n) < r1);
  LMcobweb(0.25,  4, r(n));
  TxT = ['   At r = r0 = ', num2str(r0)];
  fprintf([TxT, ', a transcritical bifurcation occurs. \n'])
end
if (r1 < r(n))*(r(n) < r2);
  LMcobweb(xa, 8, r(n));
  TxT = ['   At r = r1 = ', num2str(r1)];
  fprintf([TxT, ', a period 2 solution appears. \n'])
end
if (r2 < r(n))*(r(n) < r3);
  LMcobweb(xa, 12, r(n));
  TxT = ['   At r = r2 ~ ', num2str(r2)];
  fprintf([TxT, ', a period doubling occurs. \n'])
end
if (r3 < r(n))*(r(n) < r4);
  LMcobweb(xa, 12, r(n));
  TxT = ['   At r = r3 ~ ', num2str(r3)];
  fprintf([TxT, ', a period doubling occurs. \n'])
end
if (abs(r(n)-rinf) < 1e-4);
  LMcobweb(xa, 50, r(n));
  TxT = ['   At r = r_inf ~ ', num2str(rinf)];
  fprintf([TxT, ', chaos occurs. \n'])
end
if (abs(r(n)-rper3) < 1e-5);
  LMcobweb(xa, 50, r(n));
  TxT = ['   At r = r_per3 ~ ', num2str(rinf)];
  fprintf([TxT, ', a period 3 solution occurs. \n'])
end
axis([0 1 0 1])
%
% Now advance or go back.
%
  y = input('   --- Return to go forward, 0 to go backwards: ');
  if isempty(y); n = n+1; else; n=n-1; end;
  if n > N; GO = -1; end
  if n < 1; n = n+1; end
  hold off
end
%
% ============================================================= 65
%
% ============================================================= 65
function y = logimap(x, r, n) % Map; x = array; r = scalar.
%                             % n = iterate #
% ------------------------------------------------------------- 65
y = r*x.*(1-x);
if nargin < 3; n=1;  end;
if n > 1.1; y = logimap(y, r, n-1); end
%
% ============================================================= 65
%
% ============================================================= 65
function LMcobweb(xi, N, r) % Plots N steps of the cobweb for
%                           % the logimap, starting at xi.
% xi can be a vector.
% Assumes N > 0;   0 <= r <= 4;   and   0 <= xi < = 1.
% Assumes "hold on" in figure when called.
% ------------------------------------------------------------- 65
LiW = 'LineWidth';   LW = 2;   Ns = length(xi);
for m=1:Ns
  x = xi(m);   y = logimap(x, r);
  plot([x, x], [0, y], '-g', LiW, LW)
  for n=2:N
    plot([x, y], [y, y], '-g', LiW, LW);
    x = y;   y = logimap(x, r);
    plot([x, x], [x, y], '-g', LiW, LW)
  end
  plot([x, y], [y, y], '-g', LiW, LW)
end
%
% ============================================================= 65
%
% ============================================================= 65
function y = LMpremap(xi, r) % Find the points y that get
%                            % mapped to xi by the logimap.
% xi can be a vector.
% Assumes 0 <= r <= 4;   and   0 <= xi < = 1.
% ------------------------------------------------------------- 65
I1 = find(xi < r/4);   I2 = find(xi == r/4);   y = [];
% Note: no pre-image for xi > r/4, and just one for xi = r/4;
if ~isempty(I1);   y = sqrt(1/4-xi(I1)/r);
                   y = [0.5-y, 0.5+y];   end
if ~isempty(I2);   y = [y, 0.5];         end
% Prune out duplicates.
y = sort(y);  GO = 1;
while GO > 0;
  L = length(y);
  I = find(y(1:L-1) == y(2:L));
  if isempty(I); GO = -1;
  else  y(I) = [];
  end
end
% ============================================================= 65
%
% ============================================================= 65
% EOF
