function logimap3_2preim
% ============================================================= 65
% function logimap3_preim                            2021/11/29
% Evolution of the logistic map y = r*x*(1-x) as r grows, with
% f2 preimages of x=1/2, to illustrate when the iterates start
% to have multiple extrema. Recall:
% r = r0 = 1.           Transcritical bifurcation, with new
%                       fixed point at x = 1-1/r occurs.
% r = r1 = 3.           Period 2 solution appears;
%                       x = 1-1/r loses stability.
% r = r2 ~ 3.44948...   Period doubling;
%   = 1+sqrt(6)         period 4 solution appears;
% r = r3 ~ 3.54409...   Period doubling;
%                       period 8 solution appears;
% r_inf ~ 3.569945...
%
% Three cycle starts at r = 1 + 2*sqrt(2) ~ 3.828427...
% ============================================================= 65
LiW = 'LineWidth';   LW = 2;   FtS = 'FontSize';
x = (0:100)/100;
r = [1.5, 2.2, 3.25];
% -------------------------------- Start.
N   = length(r);   n=1;   GO = 1;
%
while GO > 0;
  y = logimap(x, r(n));
  plot(x, y, '-b', LiW, LW); hold on; grid on; zoom on;
  plot([0, 1], [0, 1], '-r', LiW, LW);
  TxT = 'Map and 0.5 preimages for r = ';
  title(['{\bf ', TxT, num2str(r(n)), '}'], FtS, 16)
  xlabel('{\bf x}', FtS, 18)
  ylabel('{\bf y}', FtS, 18, 'Rotation', 0)
  y1 = LMpremap(0.5, r(n));
  y2 = LMpremap(y1, r(n));
%
% Here goes the various alternatives!
%
if ~isempty(y2)
  LMcobweb(y2, 2, r(n));
  TxT = ['   For r = ', num2str(r(n))];
  fprintf([TxT, ', points with f(f(x)) = 0.5 exist. \n'])
end
axis([0 1 0 1])
%
% Now advance or go back.
%
  y = input('   --- Return to go forward, 0 to go backwards: ');
  if isempty(y); n = n+1; else; n=n-1; end;
  if n > N; GO = -1; end
  if n < 1; n = n+1; end
  hold off
end
%
% ============================================================= 65
%
% ============================================================= 65
function y = logimap(x, r, n) % Map; x = array; r = scalar.
%                             % n = iterate #
% ------------------------------------------------------------- 65
y = r*x.*(1-x);
if nargin < 3; n=1;  end;
if n > 1.1; y = logimap(y, r, n-1); end
%
% ============================================================= 65
%
% ============================================================= 65
function LMcobweb(xi, N, r) % Plots N steps of the cobweb for
%                           % the logimap, starting at xi.
% xi can be a vector.
% Assumes N > 0;   0 <= r <= 4;   and   0 <= xi < = 1.
% Assumes "hold on" in figure when called.
% ------------------------------------------------------------- 65
LiW = 'LineWidth';   LW = 2;   Ns = length(xi);
for m=1:Ns
  x = xi(m);   y = logimap(x, r);
  plot([x, x], [0, y], '-g', LiW, LW)
  for n=2:N
    plot([x, y], [y, y], '-g', LiW, LW);
    x = y;   y = logimap(x, r);
    plot([x, x], [x, y], '-g', LiW, LW)
  end
  plot([x, y], [y, y], '-g', LiW, LW);
  plot(y, y, '*k', 'MarkerSize', 15);
end
%
% ============================================================= 65
%
% ============================================================= 65
function y = LMpremap(xi, r) % Find the points y that get
%                            % mapped to xi by the logimap.
% xi can be a vector.
% Assumes 0 <= r <= 4;   and   0 <= xi < = 1.
% ------------------------------------------------------------- 65
I1 = find(xi < r/4);   I2 = find(xi == r/4);   y = [];
% Note: no pre-image for xi > r/4, and just one for xi = r/4;
if ~isempty(I1);   y = sqrt(1/4-xi(I1)/r);
                   y = [0.5-y, 0.5+y];   end
if ~isempty(I2);   y = [y, 0.5];         end
% Prune out duplicates.
y = sort(y);  GO = 1;
while GO > 0;
  L = length(y);
  I = find(y(1:L-1) == y(2:L));
  if isempty(I); GO = -1;
  else  y(I) = [];
  end
end
% ============================================================= 65
%
% ============================================================= 65
% EOF
