function logimap5_evolf3
% ============================================================= 65
% function logimap5_evolf3                           2021/11/29
% Evolution of the second iterate of the logistic map
%     y = r*x*(1-x) as r grows,
% to illustrate the period solution as a new fixed point for
% f3. Recall:
% r = r0 = 1.           Transcritical bifurcation, with new
%                       fixed point at x = 1-1/r occurs.
% r = r1 = 3.           Period 2 solution appears;
%                       x = 1-1/r loses stability.
% r = r2 ~ 3.44948...   Period doubling;
%   = 1+sqrt(6)         period 4 solution appears;
% r = r3 ~ 3.54409...   Period doubling;
%                       period 8 solution appears;
% r_inf ~ 3.569945...
%
% Three cycle starts at r = 1 + 2*sqrt(2) ~ 3.828427...
% ============================================================= 65
LiW = 'LineWidth';   LW = 2;   FtS = 'FontSize';
x = (0:1000)/1000;
r = [0.75, 1.5, 2, 2.8, 3.2, 3.5, 3.828, 1 + 2*sqrt(2), 3.9];
% -------------------------------- Start.
N   = length(r);   n=1;   GO = 1;
%
while GO > 0;
  y = logimap(x, r(n), 3);
  plot(x, y, '-b', LiW, LW); hold on; grid on; zoom on;
  plot([0, 1], [0, 1], '-r', LiW, LW);
  TxT = 'Third iterate for r = ';
  title(['{\bf ', TxT, num2str(r(n)), '}'], FtS, 18)
  xlabel('{\bf x}', FtS, 18)
  ylabel('{\bf y}', FtS, 18, 'Rotation', 0)
  axis([0 1 0 1])
  if (r(n) < 1 + 2*sqrt(2))*(3.78 < r(n))
    LM3cobweb(0.49, 15, r(n)) 
  end
%
% Now advance or go back.
%
  y = input('   Return to go forward, 0 to go backwards: ');
  if isempty(y); n = n+1; else; n=n-1; end;
  if n > N; GO = -1; end
  if n < 1; n = n+1; end
  hold off
end
%
% ============================================================= 65
%
% ============================================================= 65
function y = logimap(x, r, n) % Map; x = array; r = scalar.
%                             % n = iterate #
% ------------------------------------------------------------- 65
y = r*x.*(1-x);
if nargin < 3; n=1;  end;
if n > 1.1; y = logimap(y, r, n-1); end
%
% ============================================================= 65
%
% ============================================================= 65
function LM3cobweb(xi, N, r) % Plots N steps of the cobweb for
%                            % the logimap^3, starting at xi.
% xi can be a vector.
% Assumes N > 0;   0 <= r <= 4;   and   0 <= xi < = 1.
% Assumes "hold on" in figure when called.
% ------------------------------------------------------------- 65
LiW = 'LineWidth';   LW = 2;   Ns = length(xi);
for m=1:Ns
  x = xi(m);   y = logimap(x, r, 3);
  plot([x, x], [0, y], '-g', LiW, LW)
  for n=2:N
    plot([x, y], [y, y], '-g', LiW, LW);
    x = y;   y = logimap(x, r, 3);
    plot([x, x], [x, y], '-g', LiW, LW)
  end
  plot([x, y], [y, y], '-g', LiW, LW)
end
%
% ============================================================= 65
%
% ============================================================= 65
function y = LMpremap(xi, r) % Find the points y that get
%                            % mapped to xi by the logimap.
% xi can be a vector.
% Assumes 0 <= r <= 4;   and   0 <= xi < = 1.
% ------------------------------------------------------------- 65
I1 = find(xi < r/4);   I2 = find(xi == r/4);   y = [];
% Note: no pre-image for xi > r/4, and just one for xi = r/4;
if ~isempty(I1);   y = sqrt(1/4-xi(I1)/r);
                   y = [0.5-y, 0.5+y];   end
if ~isempty(I2);   y = [y, 0.5];         end
% Prune out duplicates.
y = sort(y);  GO = 1;
while GO > 0;
  L = length(y);
  I = find(y(1:L-1) == y(2:L));
  if isempty(I); GO = -1;
  else  y(I) = [];
  end
end
% ============================================================= 65
%
% ============================================================= 65
% EOF
